﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace MP3Player
{
    public partial class VolumeControl : UserControl
    {
        /// <summary>
        /// Is raised when the user changed the volume
        /// </summary>
        public event EventHandler<EventArgs> VolumeChanged;
        private int _volume;

        /// <summary>
        /// Gets or sets the Volume (0-100)
        /// </summary>
        public int Volume
        {
            get
            {
                return _volume;
            }
            set
            {
                _volume = value;
                Invalidate();
            }
        }

        public VolumeControl()
        {
            InitializeComponent();
            SetStyle(ControlStyles.UserMouse | ControlStyles.SupportsTransparentBackColor | ControlStyles.UserPaint, true);
            this.UpdateStyles();
        }

        private void VolumeControl_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            float fillWidth = 10 + (Width - 14) * (Volume / 100f);
            float fillHeight = 2 + (Height - 6) * (1f - Volume / 100f);
        
            PointF[] points = new PointF[4];
            points[0] = new PointF(10, Height - 2);
            points[1] = new PointF(fillWidth, Height - 2);
            points[2] = new PointF(fillWidth, fillHeight);
            points[3] = new PointF(10, Height - 4);
            //paint gradient
            using (LinearGradientBrush b = new LinearGradientBrush(new PointF(10, Height - 2), new PointF(Width - 4, Height - 2), Color.Red, Color.Green))
            {
                Color[] myColors = { Color.LightGreen, Color.Gold, Color.Red};
                float[] myPositions = { 0.0f, 0.8f, 1.0f };
                ColorBlend myBlend = new ColorBlend();
                myBlend.Colors = myColors;
                myBlend.Positions = myPositions;
                b.InterpolationColors = myBlend;
                g.FillPolygon((Brush)b, points);
            }

            //paint border;
            points[1] = new PointF(Width - 4, Height - 2);
            points[2] = new PointF(Width - 4, 2);
            using (Pen p = new Pen(Color.FromKnownColor(KnownColor.ControlLightLight)) { LineJoin = System.Drawing.Drawing2D.LineJoin.Round})
            {
                g.DrawLines(p, new PointF[3] { points[0], points[1], points[2] });
            }
            using (Pen p = new Pen(Color.FromKnownColor(KnownColor.ControlDark)) { LineJoin = System.Drawing.Drawing2D.LineJoin.Round})
            {
                g.DrawLines(p, new PointF[3] { points[2], points[3], points[0] });
            }
            TextRenderer.DrawText(g,_volume.ToString() + "%", this.Font,  new Point(0, 0),Color.FromKnownColor(KnownColor.ControlDark));
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int newvolume = (int)(100f * (e.X - 10) / (Width - 14));
                newvolume = Math.Max(newvolume, 0);
                newvolume = Math.Min(newvolume, 100);
                Volume = newvolume;
                RaiseVolumeChangedEvent();
            }
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            OnMouseMove(e);
        }

        protected virtual void RaiseVolumeChangedEvent()
        {
            EventHandler<EventArgs> handler = VolumeChanged;

            // Event will be null if there are no subscribers
            if (handler != null)
            {
                handler(this, new EventArgs());
            }
        }
    }
}
